local SimpleCooldownBars = CreateFrame("Frame")
local AceGUI = LibStub("AceGUI-3.0")
if not _G["SimpleCooldownBars"] then
	_G["SimpleCooldownBars"] = SimpleCooldownBars
end
SimpleCooldownBars.bars = {}
SimpleCooldownBars.spellicons = setmetatable({}, {__index=function(t,v) t[v] = (select(3, GetSpellInfo(v))) return t[v] end})
local function CopyTable(t)
	local a = {}
	for key, val in pairs(t) do
		a[key] = type(val) == "table" and CopyTable(val) or val
	end
	setmetatable(a, getmetatable(t))
	return a
end
local function chainprint(t, ...)
	local a={...}
	local pkey
	for index,key in ipairs(a) do
		print(pkey or "root".."->"..tostring(t))
		if t[key] then
			t = t[key]
			pkey = key
		else
			print(key.." is nil")
			break
		end
	end
end
local function round(num, places)
	return tonumber(string.format("%."..(places or 0).."f",num))
end
local findcache
local function tfind(t, skey, string, nod)
	if not nod then findcache = {} end
	if not string then string = "" end
	for key, data in pairs(t) do
		if key == skey then
			print(string.."."..key, data)
		end
		if type(data) == "table" and not findcache[tostring(data)] then
			findcache[tostring(data)] = true
			tfind(data, skey, string.."."..key, true)
		end
	end
end
local defaults = {
	spells={}
}
if not SCB_DB then SCB_DB = {profiles = {},data = {}} end
SimpleCooldownBars.configs = setmetatable({}, {__index=function(t,v)
	if not SimpleCooldownBars.frame then return end
	local f = SimpleCooldownBars.frame
	local group = AceGUI:Create("InlineGroup")
	f:AddChild(group)
	group:SetFullWidth(true)
	f:DoLayout()
	group:SetAutoAdjustHeight(true)
	group:SetLayout("Flow")
	local height = AceGUI:Create("EditBox")
	group:AddChild(height)
	height:SetLabel("Height")
	height:SetRelativeWidth(0.49)
	group.h = height
	local width = AceGUI:Create("EditBox")
	group:AddChild(width)
	width:SetLabel("Width")
	width:SetRelativeWidth(0.49)
	group.w = width
	local lock = AceGUI:Create("CheckBox")
	group:AddChild(lock)
	--chainprint(SCB_DB, "data", profile, "spells", spell, "lock")
	lock:SetLabel("Lock movement")
	lock:SetDescription("Disable dragging for this spell bar")
	lock:SetRelativeWidth(0.49)
	group.l = lock
	local showoff = AceGUI:Create("CheckBox")
	group:AddChild(showoff)
	showoff:SetLabel("Show while off cooldown")
	showoff:SetDescription("Shows the empty bar while the spell is not on cooldown")
	showoff:SetRelativeWidth(0.49)
	group.s = showoff
	local showgcd = AceGUI:Create("CheckBox")
	group:AddChild(showgcd)
	showgcd:SetLabel("Show Global Cooldown")
	showgcd:SetDescription("Show the global cooldown as spell cooldown")
	showgcd:SetRelativeWidth(0.49)
	group.sg = showgcd
	local icon = AceGUI:Create("CheckBox")
	group:AddChild(icon)
	icon:SetLabel("Hide icon")
	icon:SetDescription("Hide the icon for this bar")
	icon:SetRelativeWidth(0.49)
	group.i = icon
	local remove = AceGUI:Create("Button")
	remove:SetFullWidth(true)
	group:AddChild(remove)
	remove:SetText("Remove")
	group.r = remove
	group.frame:Hide()
	t[v] = group
	return t[v]
end})
local function ReleaseSelf(widget) AceGUI:Release(widget) end
local profile
SimpleCooldownBars:SetScript("OnUpdate", function() if UnitName("player") and GetRealmName() then profile = SCB_DB.profiles[UnitName("player").." - "..GetRealmName()] SimpleCooldownBars:SetScript("OnUpdate", nil) for spell in pairs(SCB_DB.data[profile].spells) do SimpleCooldownBars:CreateBar(spell) end end end)

function SimpleCooldownBars:UpdateVisibility()
	for key, frame in pairs(self.bars) do
		frame:UpdateVisibility()
	end
end
SimpleCooldownBars:SetScript("OnEvent", function(self)
	setmetatable(SCB_DB.profiles, {__index=function(t,v) t[v] = v return t[v] end})
	setmetatable(SCB_DB.data, {__index=function(t,v) t[v] = CopyTable(defaults) return t[v] end})
	SimpleCooldownBars:UnregisterEvent("ADDON_LOADED")
	SimpleCooldownBars:SetScript("OnEvent", function(self) self:UpdateVisibility() end)
	SimpleCooldownBars:RegisterEvent("SPELL_UPDATE_COOLDOWN")
end)
SimpleCooldownBars:RegisterEvent("ADDON_LOADED")

function SimpleCooldownBars:CreateBar(spell)
	SimpleCooldownBars.bars[spell] = CreateFrame("Frame", nil, UIParent)
	local data = SCB_DB.data[profile].spells[spell]
	local bar = SimpleCooldownBars.bars[spell]
	bar:SetPoint(data.point, nil, data.relative, x, y)
	bar:SetMovable(true)
	bar:EnableMouse(true)
	bar:SetScript("OnMouseDown", function(self) if not data.lock then self:StartMoving() end end)
	bar:SetScript("OnMouseUp", function(self) if not data.lock then self:StopMovingOrSizing() end end)
	bar:SetScript("OnDragStop", function(self) data.point, _, data.relative, data.x, data.y = self:GetPoint() end)
	bar:RegisterEvent("PLAYER_LOGOUT")
	bar:SetScript("OnEvent", function(self) data.point, _, data.relative, data.x, data.y = self:GetPoint() end)
	function bar:UpdateVisibility()
		local a,b,c = GetSpellCooldown(spell)
		if not a then self:Hide() return end if a == 0 or (not data.showgcd and b<=1.5) then self.showtex:Hide() if not data.showoff then self:Hide() else self:Show() end else self.showtex:Show() self:Show() end
	end
	bar.bgtex = bar:CreateTexture(nil, "BACKGROUND")
	bar.bgtex:SetAllPoints()
	bar.bgtex:SetTexture(0.6,0.6,1,0.3)
	bar.showtex = bar:CreateTexture(nil, "BACKGROUND")
	bar.showtex:SetPoint("TOPLEFT")
	bar.showtex:SetPoint("BOTTOMLEFT")
	bar.showtex:SetTexture(0.3,0.3,1,1)
	bar.text = bar:CreateFontString(nil, "ARTWORK")
	bar.text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	bar.text:SetTextColor(0.8,0.8,0.8)
	bar.text:SetText(spell)
	bar.text:SetPoint("TOPLEFT", 5, 0)
	bar.text:SetPoint("BOTTOMLEFT", 5, 0)
	bar.text:SetJustifyH("LEFT")
	bar.cdtext = bar:CreateFontString(nil, "ARTWORK")
	bar.cdtext:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	bar.cdtext:SetTextColor(0.8,0.8,0.8)
	bar.cdtext:SetPoint("TOPRIGHT", -5, 0)
	bar.cdtext:SetPoint("BOTTOMRIGHT", -5, 0)
	bar.cdtext:SetJustifyH("RIGHT")
	bar.icon = bar:CreateTexture(nil, "BACKGROUND")
	bar.icon:SetPoint("TOPRIGHT", bar, "TOPLEFT")
	bar.icon:SetPoint("BOTTOMRIGHT", bar, "BOTTOMLEFT")
	bar.icon:SetTexture(SimpleCooldownBars.spellicons[spell])
	local sh = bar.SetHeight
	function bar:SetHeight(height)
		bar.icon:SetWidth(height)
		sh(bar, height)
		bar:SetHitRectInsets(-1*height, 0, 0, 0)
	end
	bar:SetHeight(data.height)
	bar:SetWidth(data.width)
	if data.hideicon then
		bar.icon:Hide()
	end
	bar:SetScript("OnUpdate", function(self) local start, duration = GetSpellCooldown(spell) if (not start) or start == 0 or (not data.showgcd and duration<=1.5) then bar.showtex:Hide() bar.cdtext:SetText("READY") else local tr=start+duration-GetTime() bar.showtex:SetWidth(data.width*(tr/duration)) bar.cdtext:SetText(round(tr, 1).."s") end end)
	bar:UpdateVisibility()
end

function SimpleCooldownBars:CreateForSpell(input)
	if tonumber(input) and GetSpellInfo(input) then input = GetSpellInfo(tonumber(input)) end
	if not GetSpellInfo(input) then print(input.." is not a valid spell") end
	if SCB_DB.data[profile].spells[input] then
		print("A bar for "..input.." already exists")
	end
	SCB_DB.data[profile].spells[input] = {lock=false, x=0, y=0, point="CENTER", relative="CENTER", width=180, height=15, showoff = false, hideicon=false}
	self:CreateBar(input)
	self:UpdateConfig()
end

function SimpleCooldownBars:UpdateConfig()
	local c = 1
	for n, frame in pairs(SimpleCooldownBars.configs) do
		frame.frame:Hide()
	end
	for spell in pairs(SCB_DB.data[profile].spells) do
		local group = self.configs[c]
		group:SetTitle(spell)
		group.h:SetText(tostring(SCB_DB.data[profile].spells[spell].height))
		group.h:SetCallback("OnEnterPressed", function(widget, callback, text) widget:SetDisabled(true) widget:SetDisabled(false) SCB_DB.data[profile].spells[spell].height = tonumber(text) SimpleCooldownBars.bars[spell]:SetHeight(tonumber(text)) SimpleCooldownBars.bars[spell].icon:SetWidth(tonumber(text)) end)
		group.w:SetText(tostring(SCB_DB.data[profile].spells[spell].width))
		group.w:SetCallback("OnEnterPressed", function(widget, callback, text) widget:SetDisabled(true) widget:SetDisabled(false) SCB_DB.data[profile].spells[spell].width = tonumber(text) SimpleCooldownBars.bars[spell]:SetWidth(tonumber(text)) end)
		group.l:SetValue(SCB_DB.data[profile].spells[spell].lock or false)
		group.l:SetCallback("OnValueChanged", function(widget, callback, value) SCB_DB.data[profile].spells[spell].lock = value end)
		group.s:SetValue(SCB_DB.data[profile].spells[spell].showoff or false)
		group.s:SetCallback("OnValueChanged", function(widget, callback, value) SCB_DB.data[profile].spells[spell].showoff = value SimpleCooldownBars.bars[spell]:UpdateVisibility() end)
		group.sg:SetValue(SCB_DB.data[profile].spells[spell].showgcd or false)
		group.sg:SetCallback("OnValueChanged", function(widget, callback, value) SCB_DB.data[profile].spells[spell].showgcd = value SimpleCooldownBars.bars[spell]:UpdateVisibility() end)
		group.i:SetValue(SCB_DB.data[profile].spells[spell].hideicon or false)
		group.i:SetCallback("OnValueChanged", function(widget, callback, value) SCB_DB.data[profile].spells[spell].hideicon = value local a =SimpleCooldownBars.bars[spell].icon a[value and "Hide" or "Show"](a) end)
		group.r:SetCallback("OnClick", function() SCB_DB.data[profile].spells[spell] = nil SimpleCooldownBars.bars[spell]:Hide() SimpleCooldownBars.bars[spell] = nil SimpleCooldownBars:UpdateConfig() end)
		group.frame:Show()
		c=c+1
	end
end

function SimpleCooldownBars:OpenConfigGUI()
	if not self.frame then
		self.frame = AceGUI:Create("Frame")
		local frame = self.frame
		frame:SetTitle("SCB beta-1.2")
		frame:SetStatusText("SimpleCooldownBars beta-1.2 configuration GUI")
		frame:SetLayout("List")
		frame:SetCallback("OnClose", function(widget) widget:Hide() end)
		local createbargroup = AceGUI:Create("SimpleGroup")
		frame:AddChild(createbargroup)
		createbargroup:SetLayout("Flow")
		createbargroup:SetFullWidth(true)
		local eb = AceGUI:Create("EditBox")
		createbargroup:AddChild(eb)
		eb:SetLabel("Spell:")
		eb:SetRelativeWidth(0.5)
		eb:SetFullHeight(true)
		local b = AceGUI:Create("Button")
		createbargroup:AddChild(b)
		b:SetText("Create")
		b:SetRelativeWidth(0.49)
		b:SetFullHeight(true)
		b.text = ""
		b:SetCallback("OnClick", function(widget) if not widget.text then return end SimpleCooldownBars:CreateForSpell(widget.text) end)
		eb:SetCallback("OnEnterPressed", function(widget, callback, text) widget:SetDisabled(true) widget:SetDisabled(false) b.text = text end)
		self:UpdateConfig()
	else
		self.frame:Show()
	end
end

SlashCmdList["COOLDOWNS_GUI"] = function() SimpleCooldownBars:OpenConfigGUI() end
SLASH_COOLDOWNS_GUI1 = "/cooldowns"